@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2022, Intel Corporation
+* Copyright (c) 2022-2024, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -58,7 +58,15 @@ MOS_STATUS SfcRenderXe_Lpm_Plus_Base::SetupSfcState(
     //Set SFD Line Buffer
     VP_RENDER_CHK_NULL_RETURN(m_renderData.sfcStateParams);
 
-    VP_RENDER_CHK_STATUS_RETURN(SetLineBuffer(m_renderData.sfcStateParams->resSfdLineBuffer, m_SFDLineBufferSurfaceArray[m_scalabilityParams.curPipe]));
+    if (m_renderData.b1stPassOfSfc2PassScaling)
+    {
+        VP_RENDER_CHK_STATUS_RETURN(SetLineBuffer(m_renderData.sfcStateParams->resSfdLineBuffer, m_SFDLineBufferSurfaceArrayfor1stPassofSfc2Pass[m_scalabilityParams.curPipe]));
+    }
+    else
+    {
+        VP_RENDER_CHK_STATUS_RETURN(SetLineBuffer(m_renderData.sfcStateParams->resSfdLineBuffer, m_SFDLineBufferSurfaceArray[m_scalabilityParams.curPipe]));
+    }
+
     VP_RENDER_CHK_STATUS_RETURN(SetLineBuffer(m_renderData.sfcStateParams->resAvsLineTileBuffer, m_AVSLineTileBufferSurface));
     VP_RENDER_CHK_STATUS_RETURN(SetLineBuffer(m_renderData.sfcStateParams->resIefLineTileBuffer, m_IEFLineTileBufferSurface));
     VP_RENDER_CHK_STATUS_RETURN(SetLineBuffer(m_renderData.sfcStateParams->resSfdLineTileBuffer, m_SFDLineTileBufferSurface));
@@ -456,13 +464,14 @@ bool SfcRenderXe_Lpm_Plus_Base::IsSFCUncompressedWriteNeeded(PVP_SURFACE targetS
     }
 
     byteInpixel = targetSurface->osSurface->OsResource.pGmmResInfo->GetBitsPerPixel() >> 3;
-#endif // !EMUL
 
     if (byteInpixel == 0)
     {
         VP_RENDER_NORMALMESSAGE("surface format is not a valid format for sfc");
         return false;
     }
+#endif  // !EMUL
+
     uint32_t writeAlignInWidth  = 32 / byteInpixel;
     uint32_t writeAlignInHeight = 8;
     
@@ -472,6 +481,13 @@ bool SfcRenderXe_Lpm_Plus_Base::IsSFCUncompressedWriteNeeded(PVP_SURFACE targetS
         (targetSurface->rcSrc.left % writeAlignInWidth) ||
         ((targetSurface->rcSrc.right - targetSurface->rcSrc.left) % writeAlignInWidth))
     {
+        // full Frame Write don't need decompression as it will not hit the compressed write limitation
+        if ((targetSurface->rcSrc.bottom - targetSurface->rcSrc.top) == targetSurface->osSurface->dwHeight &&
+            (targetSurface->rcSrc.right - targetSurface->rcSrc.left) == targetSurface->osSurface->dwWidth)
+        {
+            return false;
+        }
+
         VP_RENDER_NORMALMESSAGE(
             "SFC Render Target Uncompressed write needed, \
             targetSurface->rcSrc.top % d, \
@@ -636,9 +652,10 @@ bool SfcRenderXe_Lpm_Plus_Base::IsOutputChannelSwapNeeded(MOS_FORMAT outputForma
 bool SfcRenderXe_Lpm_Plus_Base::IsCscNeeded(SFC_CSC_PARAMS &cscParams)
 {
     VP_FUNC_CALL();
-    if (m_bVdboxToSfc && m_videoConfig.codecStandard == CODECHAL_JPEG)
+
+    if (m_bVdboxToSfc && cscParams.inputFormat != cscParams.outputFormat)
     {
-        if (cscParams.inputFormat != cscParams.outputFormat)
+        if (m_videoConfig.codecStandard == CODECHAL_JPEG || cscParams.outputFormat == Format_A8R8G8B8)
         {
             return true;
         }
@@ -668,3 +685,23 @@ MOS_STATUS SfcRenderXe_Lpm_Plus_Base::AddSfcState(
     return MOS_STATUS_SUCCESS;
 }
 
+MOS_STATUS SfcRenderXe_Lpm_Plus_Base::SetMmcParams(PMOS_SURFACE renderTarget, bool isFormatMmcSupported, bool isMmcEnabled)
+{
+    if (((renderTarget->Format == Format_A16R16G16B16) ||
+        (renderTarget->Format == Format_A16B16G16R16)) &&
+        renderTarget->CompressionMode == MOS_MMC_RC)
+    {
+        m_renderData.sfcStateParams->bMMCEnable = true;
+        m_renderData.sfcStateParams->MMCMode    = MOS_MMC_RC;
+        VP_RENDER_NORMALMESSAGE("renderTarget->Format % d, m_renderData.sfcStateParams->MMCMode % d", renderTarget->Format, m_renderData.sfcStateParams->MMCMode);
+        return MOS_STATUS_SUCCESS;
+    }
+
+    if (!isFormatMmcSupported && renderTarget->bIsCompressed && renderTarget->CompressionMode == MOS_MMC_RC)
+    {
+        VP_RENDER_ASSERTMESSAGE("CCS not cleaned due to fix function not supported the Format when compression on.");
+    }
+
+    return SfcRenderBase::SetMmcParams(renderTarget, isFormatMmcSupported, isMmcEnabled);
+}
+
