@@ -55,7 +55,9 @@ MOS_STATUS VpFeatureManagerNext::Init(void* settings)
         m_policy = MOS_New(Policy, m_vpInterface);
     }
     VP_PUBLIC_CHK_NULL_RETURN(m_policy);
-
+    vp::VpPlatformInterface *vpPlatformInterface = m_vpInterface.GetHwInterface()->m_vpPlatformInterface;
+    VP_PUBLIC_CHK_NULL_RETURN(vpPlatformInterface);
+    VP_PUBLIC_CHK_STATUS_RETURN(vpPlatformInterface->InitVpFeatureSupportBits());
     VP_PUBLIC_CHK_STATUS_RETURN(RegisterFeatures());
     return m_policy->Initialize();
 }
@@ -186,6 +188,10 @@ MOS_STATUS VpFeatureManagerNext::RegisterFeatures()
     VP_PUBLIC_CHK_NULL_RETURN(p);
     m_featureHandler.insert(std::make_pair(FeatureTypeAlpha, p));
 
+    p = MOS_New(SwFilterCgcHandler, m_vpInterface);
+    VP_PUBLIC_CHK_NULL_RETURN(p);
+    m_featureHandler.insert(std::make_pair(FeatureTypeCgc, p));
+
     m_isFeatureRegistered = true;
     return MOS_STATUS_SUCCESS;
 }
@@ -218,6 +224,10 @@ VPFeatureManager::VPFeatureManager(
     {
         m_userSettingPtr = m_hwInterface->m_osInterface->pfnGetUserSettingInstance(m_hwInterface->m_osInterface);
     }
+    if (m_hwInterface)
+    {
+        m_vpUserFeatureControl = m_hwInterface->m_userFeatureControl;
+    }
 }
 
 MOS_STATUS VPFeatureManager::CheckFeatures(void * params, bool &bApgFuncSupported)
@@ -231,12 +241,6 @@ MOS_STATUS VPFeatureManager::CheckFeatures(void * params, bool &bApgFuncSupporte
     PVP_PIPELINE_PARAMS pvpParams = (PVP_PIPELINE_PARAMS)params;
     bApgFuncSupported = false;
 
-    if (!m_hwInterface->m_osInterface->apoMosEnabled)
-    {
-        VP_PUBLIC_NORMALMESSAGE("Fallback to legacy since APO mos not enabled.");
-        return MOS_STATUS_SUCCESS;
-    }
-
     // Color fill does not need to check src params.
     if (0 == pvpParams->uSrcCount)
     {
@@ -260,34 +264,11 @@ MOS_STATUS VPFeatureManager::CheckFeatures(void * params, bool &bApgFuncSupporte
         return MOS_STATUS_SUCCESS;
     }
 
-    bool isHdrNeeded = IsHdrNeeded(pvpParams->pSrc[0], pvpParams->pTarget[0]);
-    if (isHdrNeeded && IsCroppingNeeded(pvpParams->pSrc[0]))
-    {
-        VP_PUBLIC_NORMALMESSAGE("Disable APO Path for HDR Cropping");
-        return MOS_STATUS_SUCCESS;
-    }
-
     if (pvpParams->pConstriction)
     {
         return MOS_STATUS_SUCCESS;
     }
 
-    // Disable HVS Denoise in APO path.
-    if (pvpParams->pSrc[0]->pDenoiseParams                       &&
-        pvpParams->pSrc[0]->pDenoiseParams->bEnableHVSDenoise)
-    {
-        VP_PUBLIC_NORMALMESSAGE("Disable APO Path for HVS.");
-        return MOS_STATUS_SUCCESS;
-    }
-
-    // Disable FP16 mode in APO path.
-    bool bFP16 = IS_RGB64_FLOAT_FORMAT(pvpParams->pTarget[0]->Format) || IS_RGB64_FLOAT_FORMAT(pvpParams->pSrc[0]->Format);
-    if (bFP16)
-    {
-        VP_PUBLIC_NORMALMESSAGE("Disable FP16 mode in APO path.");
-        return MOS_STATUS_SUCCESS;
-    }
-
     bApgFuncSupported = true;
 
     return MOS_STATUS_SUCCESS;
@@ -422,10 +403,7 @@ bool VPFeatureManager::IsVeboxOutFeasible(
     VP_FUNC_CALL();
 
     bool    bRet = false;
-
-    // Vebox Comp Bypass is on by default
-    uint32_t dwCompBypassMode = VP_COMP_BYPASS_DISABLED;
-    uint32_t customValue    = VP_COMP_BYPASS_ENABLED;
+    bool disableVeboxOutput = false;
 
     VP_PUBLIC_CHK_NULL_NO_STATUS(params);
     VP_PUBLIC_CHK_NULL_NO_STATUS(params->pSrc[0]);
@@ -433,15 +411,10 @@ bool VPFeatureManager::IsVeboxOutFeasible(
 
     // Read user feature key to get the Composition Bypass mode
     // Vebox Comp Bypass is on by default
-    ReadUserSetting(
-        m_userSettingPtr,
-        dwCompBypassMode,
-        __VPHAL_BYPASS_COMPOSITION,
-        MediaUserSetting::Group::Sequence,
-        customValue,
-        true);
+    VP_PUBLIC_CHK_NULL_NO_STATUS(m_vpUserFeatureControl);
+    disableVeboxOutput  = m_vpUserFeatureControl->IsVeboxOutputDisabled();
 
-    if (dwCompBypassMode != VP_COMP_BYPASS_DISABLED                            &&
+    if (!disableVeboxOutput                                                    &&
         params->uDstCount ==1                                                  &&
         SAME_SIZE_RECT(params->pSrc[0]->rcSrc, params->pSrc[0]->rcDst)         &&
         RECT1_CONTAINS_RECT2(params->pSrc[0]->rcMaxSrc, params->pSrc[0]->rcSrc) &&
@@ -509,7 +482,7 @@ bool VPFeatureManager::IsVeboxRTFormatSupport(
 
     if ((nullptr == pSrcSurface) || (nullptr == pRTSurface))
     {
-        VPHAL_RENDER_ASSERTMESSAGE(" invalid surface");
+        VP_RENDER_ASSERTMESSAGE(" invalid surface");
         return false;
     }
 
@@ -545,8 +518,8 @@ bool VPFeatureManager::IsVeboxSupported(PVP_PIPELINE_PARAMS params)
 {
     VP_FUNC_CALL();
 
-    VPHAL_RENDER_CHK_NULL_NO_STATUS(params);
-    VPHAL_RENDER_CHK_NULL_NO_STATUS(params->pSrc[0]);
+    VP_RENDER_CHK_NULL_NO_STATUS(params);
+    VP_RENDER_CHK_NULL_NO_STATUS(params->pSrc[0]);
 
     if ((nullptr != params->pSrc[0]->pDenoiseParams && true == params->pSrc[0]->pDenoiseParams->bEnableLuma) ||
         (nullptr != params->pSrc[0]->pProcampParams && true == params->pSrc[0]->pProcampParams->bEnabled) ||
@@ -587,21 +560,17 @@ bool VPFeatureManager::IsSfcOutputFeasible(PVP_PIPELINE_PARAMS params)
     bool                        disableSFC = false;
     VP_POLICY_RULES             rules = {};
 
-    VPHAL_RENDER_CHK_NULL_NO_STATUS(params);
-    VPHAL_RENDER_CHK_NULL_NO_STATUS(params->pTarget[0]);
+    VP_RENDER_CHK_NULL_NO_STATUS(params);
+    VP_RENDER_CHK_NULL_NO_STATUS(params->pTarget[0]);
 
     if (MEDIA_IS_SKU(m_hwInterface->m_skuTable, FtrSFCPipe))
     {
         // Read user feature key to Disable SFC
-        ReadUserSetting(
-            m_userSettingPtr,
-            disableSFC,
-            __VPHAL_VEBOX_DISABLE_SFC,
-            MediaUserSetting::Group::Sequence);
-
+        VP_PUBLIC_CHK_NULL_NO_STATUS(m_vpUserFeatureControl);
+        disableSFC = m_vpUserFeatureControl->IsSfcDisabled();
         if (disableSFC)
         {
-            VPHAL_RENDER_NORMALMESSAGE("SFC is disabled.");
+            VP_RENDER_NORMALMESSAGE("SFC is disabled.");
             bRet = false;
             return bRet;
         }
@@ -610,14 +579,14 @@ bool VPFeatureManager::IsSfcOutputFeasible(PVP_PIPELINE_PARAMS params)
     // params->pSrc[0] == nullptr is valid for color fill case on SFC.
     if (params->pSrc[0] && !IsVeboxInputFormatSupport(params->pSrc[0]))
     {
-        VPHAL_RENDER_NORMALMESSAGE("The input format %d is not supported by vebox.", params->pSrc[0]->Format);
+        VP_RENDER_NORMALMESSAGE("The input format %d is not supported by vebox.", params->pSrc[0]->Format);
         bRet = false;
         return bRet;
     }
 
     if (params->pTarget[0] && !IsOutputFormatSupported(params->pTarget[0]))
     {
-        VPHAL_RENDER_NORMALMESSAGE("The output format %d is not supported by vebox.", params->pSrc[0]->Format);
+        VP_RENDER_NORMALMESSAGE("The output format %d is not supported by vebox.", params->pSrc[0]->Format);
         bRet = false;
         return bRet;
     }
@@ -695,7 +664,7 @@ bool VPFeatureManager::IsSfcOutputFeasible(PVP_PIPELINE_PARAMS params)
         OUT_OF_BOUNDS(params->pTarget[0]->dwWidth, dwTargetMinWidth, dwSfcMaxWidth) ||
         OUT_OF_BOUNDS(params->pTarget[0]->dwHeight, dwTargetMinHeight, dwSfcMaxHeight))
     {
-        VPHAL_RENDER_NORMALMESSAGE("Surface dimensions not supported by SFC Pipe");
+        VP_RENDER_NORMALMESSAGE("Surface dimensions not supported by SFC Pipe");
         bRet = false;
         return bRet;
     }
@@ -703,7 +672,7 @@ bool VPFeatureManager::IsSfcOutputFeasible(PVP_PIPELINE_PARAMS params)
     {
         if (params->pSrc[0]->Rotation != VPHAL_ROTATION_IDENTITY)
         {
-            VPHAL_RENDER_NORMALMESSAGE("Interlaced scaling cannot support rotate or mirror by SFC pipe.");
+            VP_RENDER_NORMALMESSAGE("Interlaced scaling cannot support rotate or mirror by SFC pipe.");
             bRet = false;
             return bRet;
         }
@@ -713,7 +682,7 @@ bool VPFeatureManager::IsSfcOutputFeasible(PVP_PIPELINE_PARAMS params)
             params->pSrc[0]->rcDst.left != 0 ||
             params->pSrc[0]->rcDst.top  != 0)
         {
-            VPHAL_RENDER_NORMALMESSAGE("Interlaced scaling cannot support offset by SFC pipe.");
+            VP_RENDER_NORMALMESSAGE("Interlaced scaling cannot support offset by SFC pipe.");
             bRet = false;
             return bRet;
         }
@@ -726,7 +695,7 @@ bool VPFeatureManager::IsSfcOutputFeasible(PVP_PIPELINE_PARAMS params)
     if ((params->pSrc[0]->Rotation > VPHAL_ROTATION_IDENTITY && params->pSrc[0]->Rotation != VPHAL_MIRROR_HORIZONTAL) &&
         params->pTarget[0]->TileType != MOS_TILE_Y)
     {
-        VPHAL_RENDER_NORMALMESSAGE("non TileY output mirror and rotation not supported by SFC Pipe.");
+        VP_RENDER_NORMALMESSAGE("non TileY output mirror and rotation not supported by SFC Pipe.");
         bRet = false;
         return bRet;
     }
@@ -768,7 +737,7 @@ bool VPFeatureManager::IsSfcOutputFeasible(PVP_PIPELINE_PARAMS params)
     if ((fScaleX < minRatio) || (fScaleX > maxRatio) ||
         (fScaleY < minRatio) || (fScaleY > maxRatio))
     {
-        VPHAL_RENDER_NORMALMESSAGE("Scaling factor not supported by SFC Pipe.");
+        VP_RENDER_NORMALMESSAGE("Scaling factor not supported by SFC Pipe.");
         bRet = false;
         return bRet;
     }
@@ -816,7 +785,7 @@ bool VPFeatureManager::IsRGBOutputFormatSupported(PVPHAL_SURFACE outSurface)
 {
     if (nullptr == outSurface)
     {
-        VPHAL_RENDER_ASSERTMESSAGE(" invalid outputsurface");
+        VP_RENDER_ASSERTMESSAGE(" invalid outputsurface");
         return false;
     }
 
@@ -834,7 +803,7 @@ bool VPFeatureManager::IsNV12P010OutputFormatSupported(PVPHAL_SURFACE outSurface
 {
     if (nullptr == outSurface)
     {
-        VPHAL_RENDER_ASSERTMESSAGE(" invalid outputsurface");
+        VP_RENDER_ASSERTMESSAGE(" invalid outputsurface");
         return false;
     }
 
@@ -856,7 +825,7 @@ bool VPFeatureManager::IsOutputFormatSupported(PVPHAL_SURFACE outSurface)
     VP_FUNC_CALL();
     if (nullptr == outSurface)
     {
-        VPHAL_RENDER_ASSERTMESSAGE(" invalid outputsurface");
+        VP_RENDER_ASSERTMESSAGE(" invalid outputsurface");
         return false;
     }
 
@@ -879,7 +848,7 @@ bool VPFeatureManager::IsOutputFormatSupported(PVPHAL_SURFACE outSurface)
     }
     else
     {
-        VPHAL_RENDER_NORMALMESSAGE("Unsupported Render Target Format '0x%08x' for SFC Pipe.", outSurface->Format);
+        VP_RENDER_NORMALMESSAGE("Unsupported Render Target Format '0x%08x' for SFC Pipe.", outSurface->Format);
         ret = false;
     }
 
@@ -1012,7 +981,7 @@ MOS_STATUS VPFeatureManager::RectSurfaceAlignment(
         (pSurface->dwWidth    == 0)                      ||
         (pSurface->dwHeight   == 0))
     {
-        VPHAL_RENDER_ASSERTMESSAGE("Surface Parameter is invalid.");
+        VP_RENDER_ASSERTMESSAGE("Surface Parameter is invalid.");
         eStatus = MOS_STATUS_INVALID_PARAMETER;
     }
 
