@@ -72,6 +72,7 @@ MOS_STATUS VpRenderKernelObj::GetKernelEntry(Kdll_CacheEntry &entry)
     entry.iFilterSize   = 2;
     entry.pFilter       = nullptr;
     entry.iSize         = m_kernelSize;
+    entry.iPaddingSize  = m_kernelPaddingSize; 
     entry.pBinary       = (uint8_t *)m_kernelBinary;
     return MOS_STATUS_SUCCESS;
 }
@@ -237,7 +238,7 @@ MOS_STATUS VpRenderKernelObj::SetupStatelessBuffer()
     return MOS_STATUS_SUCCESS;
 }
 
-MOS_STATUS VpRenderKernelObj::SetupStatelessBufferResource(SurfaceType surf)
+MOS_STATUS VpRenderKernelObj::SetupStatelessBufferResource(SurfaceType surf, bool isWrite)
 {
     VP_RENDER_CHK_NULL_RETURN(m_surfaceGroup);
     VP_RENDER_CHK_NULL_RETURN(m_hwInterface);
@@ -253,7 +254,7 @@ MOS_STATUS VpRenderKernelObj::SetupStatelessBufferResource(SurfaceType surf)
         VP_RENDER_CHK_STATUS_RETURN(osInterface->pfnRegisterResource(
             osInterface,
             &curSurf->osSurface->OsResource,
-            false,
+            isWrite,
             true));
         m_statelessArray.insert(std::make_pair(surf, ui64GfxAddress));
     }
@@ -619,34 +620,6 @@ finish:
     return eStatus;
 }
 
-void VpRenderKernelObj::PrintCurbe(uint8_t* pCurbe, uint32_t curbeLength)
-{
-#if (_DEBUG || _RELEASE_INTERNAL)
-    std::string curbeString = "";
-    if (curbeLength > 0 && pCurbe != nullptr)
-    {
-        for (uint32_t i = 0; i < curbeLength; ++i)
-        {
-            uint8_t           iData = pCurbe[i];
-            std::stringstream hex;
-            if (i % 8 == 0)
-            {
-                hex << "offset " << std::setw(3) << std::setfill('0') << i << ": ";
-            }
-            hex << "0x" << std::hex << std::setfill('0') << std::setw(2) << static_cast<int>(iData) << " ";
-            curbeString += hex.str();
-            if (i % 8 == 7)
-            {
-                curbeString += "\n";
-            }
-        }
-    }
-    VP_RENDER_VERBOSEMESSAGE("Curbe Data Length = %d, Content = \n%s",
-        curbeLength,
-        curbeString.c_str());
-#endif
-}
-
 void VpRenderKernelObj::DumpSurface(VP_SURFACE* pSurface, PCCHAR fileName)
 {
     uint8_t* pData;
